#ifndef LINUX_SKBUFF_H_BACKPORT
#define LINUX_SKBUFF_H_BACKPORT

#include_next <linux/skbuff.h>

#define CHECKSUM_PARTIAL CHECKSUM_HW
#define CHECKSUM_COMPLETE CHECKSUM_HW

static inline void skb_set_queue_mapping(struct sk_buff *skb, int mapping)
{
	skb->priority = mapping;
}

/*
 * skb_checksum_none_assert - make sure skb ip_summed is CHECKSUM_NONE
 * @skb: skb to check
 *
 * fresh skbs have their ip_summed set to CHECKSUM_NONE.
 * Instead of forcing ip_summed to CHECKSUM_NONE, we can
 * use this helper, to document places where we make this assertion.
 *
 * Adding missing function in RHEL5.6 kernel.
 */
static inline void skb_checksum_none_assert(struct sk_buff *skb)
{
#ifdef DEBUG
    BUG_ON(skb->ip_summed != CHECKSUM_NONE);
#endif
}

#endif
#ifndef __BACKPORT_LINUX_SKBUFF_H_TO_2_6_21__
#define __BACKPORT_LINUX_SKBUFF_H_TO_2_6_21__

#include_next <linux/skbuff.h>

#define transport_header h.raw
#define network_header nh.raw

static inline int skb_csum_unnecessary(const struct sk_buff *skb)
{
	return skb->ip_summed & CHECKSUM_UNNECESSARY;
}

static inline void skb_record_rx_queue(struct sk_buff *skb, u16 rx_queue)
{
}


#define skb_queue_walk_safe(queue, skb, tmp)				\
		for (skb = (queue)->next, tmp = skb->next;		\
		     skb != (struct sk_buff *)(queue);			\
		     skb = tmp, tmp = skb->next)

#endif
