#ifndef __ETHTOOL__BACKPORT_H_TO_2_6_30___
#define __ETHTOOL__BACKPORT_H_TO_2_6_30___

#include_next <linux/ethtool.h>

/**
 * enum ethtool_phys_id_state - indicator state for physical identification
 * @ETHTOOL_ID_INACTIVE: Physical ID indicator should be deactivated
 * @ETHTOOL_ID_ACTIVE: Physical ID indicator should be activated
 * @ETHTOOL_ID_ON: LED should be turned on (used iff %ETHTOOL_ID_ACTIVE
 *      is not supported)
 * @ETHTOOL_ID_OFF: LED should be turned off (used iff %ETHTOOL_ID_ACTIVE
 *      is not supported)
 */
enum ethtool_phys_id_state {
    ETHTOOL_ID_INACTIVE,
    ETHTOOL_ID_ACTIVE,
    ETHTOOL_ID_ON,
    ETHTOOL_ID_OFF
};

/* boolean flags controlling per-interface behavior characteristics.
 * When reading, the flag indicates whether or not a certain behavior
 * is enabled/present.  When writing, the flag indicates whether
 * or not the driver should turn on (set) or off (clear) a behavior.
 *
 * Some behaviors may read-only (unconditionally absent or present).
 * If such is the case, return EINVAL in the set-flags operation if the
 * flag differs from the read-only value.
 *
 * Adding missing enums for ethtool_flags in 2.6.30 kernel.
 */
enum additional_ethtool_flags {
    ETH_FLAG_TXVLAN         = (1 << 7),     /* TX VLAN offload enabled */
    ETH_FLAG_RXVLAN         = (1 << 8),     /* RX VLAN offload enabled */
};

#define PORT_DA                 0x05
#define IPV4_FLOW               0x10    /* hash only */
#define IPV6_FLOW               0x11    /* hash only */

#define SUPPORTED_Backplane             (1 << 16)
#define SUPPORTED_1000baseKX_Full       (1 << 17)
#define SUPPORTED_10000baseKX4_Full     (1 << 18)
#define SUPPORTED_10000baseKR_Full      (1 << 19)
#define SUPPORTED_10000baseR_FEC        (1 << 20)
#define PORT_OTHER              0xff

#endif /* __ETHTOOL__BACKPORT_H_TO_2_6_30___ */
