#ifndef __NETDEVICE__BACKPORT_H_TO_2_6_32___
#define __NETDEVICE__BACKPORT_H_TO_2_6_32___

#include_next <linux/netdevice.h>

static inline int netdev_queue_numa_node_read(const struct netdev_queue *q)
{
#if defined(CONFIG_XPS) && defined(CONFIG_NUMA)
    return q->numa_node;
#else
    return NUMA_NO_NODE;
#endif
}

/**
 *      netif_set_real_num_rx_queues - set actual number of RX queues used
 *      @dev: Network device
 *      @rxq: Actual number of RX queues
 *
 *     This function actully sets the real_num_rx_queues field in struct
 *     net_device. Since real_num_rx_queues field is not present in net_device
 *     structure in 2.6.32 kernel making this function to set that field is not
 *     possible. Hence adding this function to avoid changes in driver source
 *     code and making this function to always return success.
 */
static inline int netif_set_real_num_rx_queues(struct net_device *dev,
	unsigned int rxq)
{
    return 0;
}

static inline int netif_set_real_num_tx_queues(struct net_device *dev,
	unsigned int rxq)
{
    return 0;
}

#define NETIF_F_RXHASH          (1 << 28) /* Receive hashing offload */

#define netdev_for_each_mc_addr(mclist, dev) \
    for (mclist = dev->mc_list; mclist; mclist = mclist->next)

static inline const char *netdev_name(const struct net_device *dev)
{
    if (dev->reg_state != NETREG_REGISTERED)
	return "(unregistered net_device)";
    return dev->name;
}

/* debugging and troubleshooting/diagnostic helpers. */
/*
 * netdev_printk() - Log message with interface name, driver name, bus ID.
 * @level: severity level -- e.g., KERN_INFO
 * @netdev: net_device pointer
 * @format: as with printk
 * @args: as with printk
 */
#define netdev_printk(level, netdev, format, arg...)    \
    printk(level "%s: " format , netdev_name(netdev), ## arg)

#define netdev_info(netdev, format, arg...)             \
    netdev_printk(KERN_INFO, netdev, format, ## arg)

#endif /* __NETDEVICE__BACKPORT_H_TO_2_6_32___ */
