#ifndef _LINUX_NETDEVICE_BACKPORT_TO_2_6_16
#define _LINUX_NETDEVICE_BACKPORT_TO_2_6_16

#include_next <linux/netdevice.h>

static inline int skb_checksum_help_to_2_6_18(struct sk_buff *skb)
{
        return skb_checksum_help(skb, 0);
}

#define skb_checksum_help skb_checksum_help_to_2_6_18

#undef SET_ETHTOOL_OPS
#define SET_ETHTOOL_OPS(netdev, ops) \
	(netdev)->ethtool_ops = (struct ethtool_ops *)(ops)

#define NETIF_F_TSO6    0
#define NETIF_F_LRO		32768   /* large receive offload */
#define NETIF_F_IPV6_CSUM	16	/* Can checksum TCP/UDP over IPV6 */

#define dev_get_by_name(net, name) dev_get_by_name(name)

#define for_each_netdev(a, dev) for ((dev) = dev_base; (dev) != NULL; (dev) = (dev)->next)

#define vlan_dev_real_dev(netdev) (VLAN_DEV_INFO(netdev)->real_dev)

#define NUMA_NO_NODE    (-1)

/**
 * netif_set_real_num_rx_queues - set actual number of RX queues used
 * @dev: Network device
 * @rxq: Actual number of RX queues
 *
 * This function actully sets the real_num_rx_queues field in struct
 * net_device. Since real_num_rx_queues field is not present in
 * net_device structure in 2.6.32 kernel making this function to set
 * that field is not possible. Hence adding this function to avoid changes
 * in driver source code and making this function to always return success.
 */
static inline int netif_set_real_num_rx_queues(struct net_device *dev,
		unsigned int rxq)
{
	return 0;
}

static inline int netif_set_real_num_tx_queues(struct net_device *dev,
		unsigned int rxq)
{
	return 0;
}

#define NETIF_F_RXHASH          (1 << 28) /* Receive hashing offload */

#define netdev_uc_count(netdev)                 0
#define netdev_for_each_uc_addr(ha, dev)        if (0)
#define netif_tx_wake_queue(dev)        netif_wake_queue(dev)

#define dev_addr_list   dev_mc_list

/*
 * Fake this.  uc_count is zero and netdev_for_each_uc_addr will do nothing,
 * but we still want it to compile.
 */
struct netdev_hw_addr {
	u8      *addr;
};

struct napi_struct {
	/*
	 * The poll_list must only be managed by the entity which
	 * changes the state of the NAPI_STATE_SCHED bit.  This means
	 * whoever atomically sets that bit can add this napi_struct
	 * to the per-cpu poll_list, and whoever clears that bit
	 * can remove from the list right before clearing the bit.
	 */
	struct list_head        poll_list;

	unsigned long           state;
	int                     weight;
	int                     (*poll)(struct napi_struct *, int);
#ifdef CONFIG_NETPOLL
	spinlock_t              poll_lock;
	int                     poll_owner;
#endif

	unsigned int            gro_count;

	struct net_device       *dev;
	struct list_head        dev_list;
	struct sk_buff          *gro_list;
	struct sk_buff          *skb;
};

static inline void napi_enable(struct net_device *dev)
{
	if (dev)
		netif_poll_enable(dev);
}

static inline void napi_disable(struct net_device *dev)
{
	if (dev)
		netif_poll_disable(dev);
}

/*
 * No multiqueue support, so we collapse the netdev_queue back
 * to the net_device.
 */
#define netdev_queue    net_device
#define netdev_get_tx_queue(dev, j)     (dev)
#define netif_tx_stop_queue(txq)        netif_stop_queue(txq)
#define skb_get_queue_mapping(skb)      0
#define skb_record_rx_queue(a, b)       do { ; } while (0)

#define netif_tx_start_all_queues(dev)  netif_start_queue(dev)
#define netif_tx_stop_all_queues(dev)   netif_stop_queue(dev)

#define __netdev_alloc_page(a, b)               \
	alloc_pages_node(-1, (b), 0)

#define netdev_free_page(a, b)                  \
	__free_pages((b), 0)

static inline void netif_napi_add(struct net_device *dev,
		struct napi_struct *napi,
		int (*poll)(struct net_device *, int *), int weight)
{
	BUG_ON(dev->atalk_ptr);

	INIT_LIST_HEAD(&napi->poll_list);
	napi->gro_count = 0;
	napi->gro_list = NULL;
	napi->skb = NULL;

	napi->weight = weight;
	napi->dev = dev;

	dev->poll = poll;

}
static inline void netif_napi_del(struct napi_struct *napi)
{
	struct sk_buff *skb, *next;

	if (napi->skb)
		kfree_skb(napi->skb);

	for (skb = napi->gro_list; skb; skb = next) {
		next = skb->next;
		skb->next = NULL;
		kfree_skb(skb);
	}

	napi->gro_list = NULL;
	napi->gro_count = 0;
}

static inline void napi_schedule(struct net_device *dev)
{
	netif_rx_schedule(dev);
}

static inline void compat_napi_complete(struct net_device *dev)
{
	netif_rx_complete(dev);
}
#define napi_complete(x)        compat_napi_complete(x)

static inline int napi_reschedule(struct napi_struct *napi)
{
	netif_rx_schedule(napi->dev);
	return 1;
}

static inline int __netif_tx_trylock(struct net_device *txq)
{
	return netif_tx_trylock(txq);
}

static inline void __netif_tx_unlock(struct net_device *txq)
{
	netif_tx_unlock(txq);
}
#define netif_tx_start_all_queues(dev)  netif_start_queue(dev)
#define netif_tx_stop_all_queues(dev)   netif_stop_queue(dev)
#define netif_tx_wake_queue(dev)        netif_wake_queue(dev)

static inline const char *netdev_name(const struct net_device *dev)
{
	if (dev->reg_state != NETREG_REGISTERED)
		return "(unregistered net_device)";
	return dev->name;
}

#define netdev_printk(level, netdev, format, args...)           \
        dev_printk(level, (netdev)->class_dev.dev,              \
                        "%s: " format,                          \
                        netdev_name(netdev), ##args)

#define netdev_info(dev, format, args...)                       \
        netdev_printk(KERN_INFO, dev, format, ##args)

#define netdev_for_each_mc_addr(mcl, dev) \
        for (mcl = (dev)->mc_list; mcl; mcl = mcl->next)

#define NETIF_F_TSO_ECN         (SKB_GSO_TCP_ECN << NETIF_F_GSO_SHIFT)
#define NETIF_F_GRO             16384   /* Generic receive offload */

#endif
