#ifndef _NET_EVENT_H
#define _NET_EVENT_H

#include <net/dst.h>
#include <linux/notifier.h>
#include <linux/spinlock_types.h>

struct atomic_notifier_head {
	spinlock_t lock;
	struct notifier_block *head;
};

struct netevent_redirect {
	struct dst_entry *old;
	struct dst_entry *new;
};

enum netevent_notif_type {
	NETEVENT_NEIGH_UPDATE = 1, /* arg is struct neighbour ptr */
	NETEVENT_PMTU_UPDATE,	   /* arg is struct dst_entry ptr */
	NETEVENT_REDIRECT,	   /* arg is struct netevent_redirect ptr */
};

#define ATOMIC_NOTIFIER_INIT(name) {                            \
	.lock = SPIN_LOCK_UNLOCKED,        \
	.head = NULL }

#define ATOMIC_NOTIFIER_HEAD(name)                              \
	struct atomic_notifier_head name =                      \
ATOMIC_NOTIFIER_INIT(name)

static ATOMIC_NOTIFIER_HEAD(netevent_notif_chain);

static inline int atomic_notifier_chain_register(
		struct atomic_notifier_head *nh,
		struct notifier_block *n)
{
	unsigned long flags;
	int ret;

	spin_lock_irqsave(&nh->lock, flags);
	ret = notifier_chain_register(&nh->head, n);
	spin_unlock_irqrestore(&nh->lock, flags);
	return ret;
}

static inline int register_netevent_notifier(struct notifier_block *nb)
{
	int err;

	err = atomic_notifier_chain_register(&netevent_notif_chain, nb);
	return err;
}

static inline int atomic_notifier_chain_unregister(
		struct atomic_notifier_head *nh,
		struct notifier_block *n)
{
	unsigned long flags;
	int ret;

	spin_lock_irqsave(&nh->lock, flags);
	ret = notifier_chain_unregister(&nh->head, n);
	spin_unlock_irqrestore(&nh->lock, flags);
	synchronize_rcu();
	return ret;
}

static inline int unregister_netevent_notifier(struct notifier_block *nb)
{
	return atomic_notifier_chain_unregister(&netevent_notif_chain, nb);
}

extern int call_netevent_notifiers(unsigned long val, void *v);

#endif
